// 
// Copyright (C) 2007, NinjaTrader LLC <www.ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//
#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Sample MA indicator that allows you to select different MA types.
    /// </summary>
    [Description("Sample MA indicator that allows you to select different MA types.")]
    public class SampleUniversalMovingAverage : Indicator
    {
        #region Variables
		// Create a variable that stores the user's selection for a moving average 
		private UniversalMovingAverage	matype	= UniversalMovingAverage.SMA;
		private int				period	= 14;
		#endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
			// Adds a plot for the MA values to be stored in
			Add(new Plot(Color.Orange, "MA"));
			
            CalculateOnBarClose	= true;
            Overlay				= true;
            PriceTypeSupported	= true;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			// We use a switch which allows NinjaTrader to only execute code pertaining to our case
			switch (matype)
			{
				// If the matype is defined as an EMA then...
				case UniversalMovingAverage.EMA:
				{
					// Sets the plot to be equal to the EMA's plot
					Value.Set(EMA(Period)[0]);
					break;
				}
				
				// If the matype is defined as a HMA then...
				case UniversalMovingAverage.HMA:
				{
					// Sets the plot to be equal to the HMA's plot
					Value.Set(HMA(Period)[0]);
					break;
				}
				
				// If the matype is defined as a SMA then...
				case UniversalMovingAverage.SMA:
				{
					// Sets the plot to be equal to the SMA's plot
					Value.Set(SMA(Period)[0]);
					break;
				}
				
				// If the matype is defined as a WMA then...
				case UniversalMovingAverage.WMA:
				{
					// Sets the plot to be equal to the WMA's plot
					Value.Set(WMA(Period)[0]);
					break;
				}
			}
        }

        #region Properties
		// Creates the user definable parameter for the moving average type.
		[Description("Choose a Moving Average type.")]
		[Category("Parameters")]
		public UniversalMovingAverage MAType
		{
			get { return matype; }
			set { matype = value; }
		}
		
		[Description("Numbers of bars used for calculations")]
		[Category("Parameters")]
		public int Period
		{
			get { return period; }
			set { period = Math.Max(1, value); }
		}
        #endregion
    }
}

/* Creates an enum that is assigned to UniversalMovingAverage.
Basically it assigns a numerical value to each item in the list and the items can be referenced without the use
of their numerical value.
For more information on enums you can read up on it here: http://www.csharp-station.com/Tutorials/Lesson17.aspx */
public enum UniversalMovingAverage
{
	EMA,
	HMA,
	SMA,
	WMA,
}
#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private SampleUniversalMovingAverage[] cacheSampleUniversalMovingAverage = null;

        private static SampleUniversalMovingAverage checkSampleUniversalMovingAverage = new SampleUniversalMovingAverage();

        /// <summary>
        /// Sample MA indicator that allows you to select different MA types.
        /// </summary>
        /// <returns></returns>
        public SampleUniversalMovingAverage SampleUniversalMovingAverage(UniversalMovingAverage mAType, int period)
        {
            return SampleUniversalMovingAverage(Input, mAType, period);
        }

        /// <summary>
        /// Sample MA indicator that allows you to select different MA types.
        /// </summary>
        /// <returns></returns>
        public SampleUniversalMovingAverage SampleUniversalMovingAverage(Data.IDataSeries input, UniversalMovingAverage mAType, int period)
        {
            checkSampleUniversalMovingAverage.MAType = mAType;
            mAType = checkSampleUniversalMovingAverage.MAType;
            checkSampleUniversalMovingAverage.Period = period;
            period = checkSampleUniversalMovingAverage.Period;

            if (cacheSampleUniversalMovingAverage != null)
                for (int idx = 0; idx < cacheSampleUniversalMovingAverage.Length; idx++)
                    if (cacheSampleUniversalMovingAverage[idx].MAType == mAType && cacheSampleUniversalMovingAverage[idx].Period == period && cacheSampleUniversalMovingAverage[idx].EqualsInput(input))
                        return cacheSampleUniversalMovingAverage[idx];

            SampleUniversalMovingAverage indicator = new SampleUniversalMovingAverage();
            indicator.SetUp();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.MAType = mAType;
            indicator.Period = period;

            SampleUniversalMovingAverage[] tmp = new SampleUniversalMovingAverage[cacheSampleUniversalMovingAverage == null ? 1 : cacheSampleUniversalMovingAverage.Length + 1];
            if (cacheSampleUniversalMovingAverage != null)
                cacheSampleUniversalMovingAverage.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheSampleUniversalMovingAverage = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Sample MA indicator that allows you to select different MA types.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SampleUniversalMovingAverage SampleUniversalMovingAverage(UniversalMovingAverage mAType, int period)
        {
            return _indicator.SampleUniversalMovingAverage(Input, mAType, period);
        }

        /// <summary>
        /// Sample MA indicator that allows you to select different MA types.
        /// </summary>
        /// <returns></returns>
        public Indicator.SampleUniversalMovingAverage SampleUniversalMovingAverage(Data.IDataSeries input, UniversalMovingAverage mAType, int period)
        {
            return _indicator.SampleUniversalMovingAverage(input, mAType, period);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Sample MA indicator that allows you to select different MA types.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SampleUniversalMovingAverage SampleUniversalMovingAverage(UniversalMovingAverage mAType, int period)
        {
            return _indicator.SampleUniversalMovingAverage(Input, mAType, period);
        }

        /// <summary>
        /// Sample MA indicator that allows you to select different MA types.
        /// </summary>
        /// <returns></returns>
        public Indicator.SampleUniversalMovingAverage SampleUniversalMovingAverage(Data.IDataSeries input, UniversalMovingAverage mAType, int period)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.SampleUniversalMovingAverage(input, mAType, period);
        }

    }
}
#endregion
